#ifndef PLAYER_STATES_HPP
#define PLAYER_STATES_HPP

#include "state.hpp"

namespace godot {
class PlayerCharacter;
class StateMachine;
class TunnelsPlayer;

class PlayerState : public Node,
                    public IState {
    GDCLASS(PlayerState, Node);
    static void _bind_methods();
public:
    virtual void init(Node *target, StateMachine *machine) override;
    virtual StringName get_next() const override;
protected:
    float get_distance_from_player(bool sqr = false) const;
    PlayerCharacter *get_manual_character() const;
protected:
    PlayerCharacter *parent{nullptr};
    StateMachine *state_machine{nullptr};
    TunnelsPlayer *player{nullptr};
};

class FollowingPlayer : public PlayerState {
    GDCLASS(FollowingPlayer, PlayerState);
    static void _bind_methods();
public:
    virtual void _process(double delta_time) override;
    virtual StringName get_next() const override;
};

class Regroup : public PlayerState {
    GDCLASS(Regroup, PlayerState);
    static void _bind_methods();
public:
    virtual void _process(double delta_time) override;
    virtual void _exit_tree() override;
    virtual StringName get_next() const override;
private:
    Vector3 target{0.f, 0.f, 0.f};
};
}

#endif // !PLAYER_STATES_HPP
